using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace LCDAdapt
{
    public partial class NavigateForm : BaseForm
    {
        public NavigateForm()
        {
            InitializeComponent();
            this.IsMdiContainer = true;
            PopulateTreeView();            
        }

        private void PopulateTreeView()
        {
            TreeNode node = new TreeNode("Set & Send");
            MainForm mForm = new MainForm();
            mForm.SetLcdSettings = new SetLcdSettings(SetLcdSettingsMainForm);
            mForm.GetLcdSettings = new GetLcdSettings(GetLcdSettingsMainForm);
            mForm.MdiParent = this;
            node.Tag = mForm;
            treeviewNavigate.Nodes.Add(node);
            node = new TreeNode("Export/Import");
            Export_Import mExp = new Export_Import();
            mExp.SetLcdSettings = new SetLcdSettings(SetLcdSettingsExpImpForm);
            mExp.GetLcdSettings = new GetLcdSettings(GetLcdSettingsExpImpForm);
            mExp.MdiParent = this;
            node.Tag = mExp;
            treeviewNavigate.Nodes.Add(node);
            treeviewNavigate.ShowRootLines = false;

            owner = SettingsOwner.None;
        }

        private void treeviewNavigate_AfterSelect(object sender, TreeViewEventArgs e)
        {
            foreach (TreeNode node in treeviewNavigate.Nodes)
            {
                Form tmp = (Form)node.Tag;
                if (node.IsSelected)
                {
                    Point p = new Point(treeviewNavigate.Location.X, treeviewNavigate.Location.Y);

                    p.X += treeviewNavigate.Width + 4;
                    tmp.Location = p;
                    tmp.Visible = true;
                }
                else
                {
                    tmp.Visible = false;
                }
            }
        }

        #region delgates implemetation
        /// <summary>
        /// Holds the updated global LcdSettings of all forms
        /// </summary>
        private LcdSettings globalLcdSettings;
        enum SettingsOwner
        {
            None,
            MainForm,
            ExpImpForm
        }
        private SettingsOwner owner;
        /// <summary>
        /// Sets the global settings field
        /// </summary>
        /// <param name="settings"></param>
        /// <returns>
        /// true - when sets successfully
        /// false - when the ownership is already of the form trying to do the op
        /// </returns>
        bool SetLcdSettingsMainForm(LcdSettings settings)
        {
            if (!settings.Equals(globalLcdSettings) || owner == SettingsOwner.None)
            {
                globalLcdSettings = settings;
                owner = SettingsOwner.MainForm;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Sets the global settings field
        /// </summary>
        /// <param name="settings"></param>
        /// <returns>
        /// true - when sets successfully
        /// false - when the ownership is already of the form trying to do the op
        /// </returns>
        bool SetLcdSettingsExpImpForm(LcdSettings settings)
        {
            if (!settings.Equals(globalLcdSettings) || owner == SettingsOwner.None)
            {
                globalLcdSettings = settings;
                owner = SettingsOwner.ExpImpForm;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Gets the global setting field
        /// </summary>
        /// <param name="settings"></param>
        /// <returns>
        /// true- when global setting copied
        /// false - if global setting is of the ownership of the qurying form
        /// </returns>
        bool GetLcdSettingsMainForm(out LcdSettings settings)
        {
            if (owner != SettingsOwner.MainForm && owner != SettingsOwner.None)
            {
                settings = globalLcdSettings;
                return true;
            }
            //Garbage assignment
            LcdSettings tmp = new LcdSettings();
            tmp.IsValid = false;
            settings = tmp;
            return false;
        }

        /// <summary>
        /// Gets the global setting field
        /// </summary>
        /// <param name="settings"></param>
        /// <returns>
        /// true- when global setting copied
        /// false - if global setting is of the ownership of the qurying form
        /// </returns>
        bool GetLcdSettingsExpImpForm(out LcdSettings settings)
        {
            if (owner != SettingsOwner.ExpImpForm && owner != SettingsOwner.None)
            {
                settings = globalLcdSettings;
                return true;
            }
            //Garbage assignment
            LcdSettings tmp = new LcdSettings();
            tmp.IsValid = false;
            settings = tmp;
            return false;
        }
        #endregion

    }
}